from cement.core.output import OutputHandler
from rich.console import Console
from rich.table import Table
from rich.panel import Panel
from rich import print as rich_print
from rich.markup import escape

class BaseRenderer:
    def __init__(self, *args, **kwargs):
        self.run_validations(*args, **kwargs)
        self.render(*args, **kwargs)

    def run_validations(self, *args, **kwargs):
        pass

    def render(self, *args, **kwargs):
        pass


class TableRenderer(BaseRenderer):
    def run_validations(self, *args, **kwargs):
        rows = kwargs.get('rows')
        assert rows is not None
        assert isinstance(rows, list)

        headers = kwargs.get('headers')
        assert headers is not None
        assert isinstance(headers, dict)

    def render(self, *args, **kwargs):
        headers = kwargs.get('headers')
        rows = kwargs.get('rows')
        table = Table()
        for column_header in headers.values():
            table.add_column(column_header)

        for datum in rows:
            row = list()
            for col in headers.keys():
                value = datum[col]
                if isinstance(value, bool):
                    value = 'Yes' if value else 'No'
                row.append(str(value))
            table.add_row(*row)

        console = Console(highlight=False)
        console.print(table)


class RealtimeLogRenderer(BaseRenderer):
    def run_validations(self, *args, **kwargs):
        rows = kwargs.get('rows')
        assert rows is not None
        assert isinstance(rows, list)

    def render(self, *args, **kwargs):
        rows = kwargs.get('rows')
        text = ''
        for row in rows:
            text += f'[bold]{row.get("application_type")}:[/bold] {row.get("message")}\n'

        console = Console(highlight=False)
        console.print(text)


class TextRenderer(BaseRenderer):
    def render(self, *args, **kwargs):
        text = kwargs.get('custom_text')
        print(text)


class PipelineLogRenderer(BaseRenderer):
    def render(self, *args, **kwargs):
        rows = kwargs.get('rows', [])
        if not rows:
            rich_print(Panel("No log found."))
        for row in rows:
            rich_print(Panel(f'{escape(row.get("log_data"))}', title=f"[bold][red]{row.get('created_date')}[/]"))


class AkinonOutputHandler(OutputHandler):
    renderers = {
        "text": TextRenderer,
        "realtime_log": RealtimeLogRenderer,
        "table": TableRenderer,
        "pipeline_log": PipelineLogRenderer
    }

    class Meta:
        label = 'akinon_output_handler'

    def render(self, data, *args, **kwargs):
        renderer_type = kwargs.pop('renderer_type', 'table')
        is_succeed = kwargs.pop('is_succeed', False)
        if is_succeed or renderer_type == 'text':
            self.renderers.get(renderer_type)(*args, **kwargs)
        else:
            self.app.log.error(data)



import json

from cement import Controller, ex
from cement.utils.version import get_version_banner

from . import PaginationEnum
from ..core.version import get_version


VERSION_BANNER = """
CLI for Akinon Cloud Commerce %s
%s
""" % (get_version(), get_version_banner())


class Domains(Controller):
    class Meta:
        label = 'domain'
        stacked_type = 'nested'
        stacked_on = 'base'
        description = 'this is the domain controller namespace'

    @ex(
        help='Domain List Command',
        arguments=[
            PaginationEnum.ARG
        ],
    )
    def list(self):
        response = self.app.client.get_domains(
            qs={"page": getattr(self.app.pargs, PaginationEnum.KEY)}
        )
        rows = response.data.get('results', [])
        for row in rows:
            row['extra'] = json.dumps(row['extra_data'], indent=4)
        self.app.render(data=response.data,
                        rows=response.data.get('results', []),
                        headers={'pk': 'ID',
                                 'hostname': 'Hostname',
                                 'is_usable': 'Is Usable',
                                 'is_managed': 'Is Managed',
                                 'extra': 'Extra Data'},
                        is_succeed=response.is_succeed)

    @ex(
        help='Domain Create Command',
        arguments=[
            (['hostname'], {
                'help': 'Hostname',
                'action': 'store',
            }),
            (['is_managed'], {
                'help': 'Is Managed',
                'action': 'store',
            }),
        ]
    )
    def create(self):
        data = {
            'hostname': self.app.pargs.hostname,
            'is_managed': self.app.pargs.is_managed,
        }
        response = self.app.client.create_domain(data)
        if response.is_succeed:
            self.app.render({}, renderer_type='text',
                            custom_text="Domain has been created.")
        else:
            custom_text = response.data.get('non_field_errors')
            if not custom_text:
                custom_text = response.data.get('hostname')[0]
            self.app.render({}, renderer_type='text', custom_text=custom_text)


class Certificates(Controller):
    class Meta:
        label = 'certificate'
        stacked_type = 'nested'
        stacked_on = 'base'
        description = 'this is the certificate controller namespace'

    @ex(
        help="Domain's Certificate List Command",
        arguments=[
            (['domain_id'], {
                'help': 'Domain ID',
                'action': 'store',
            }),
            PaginationEnum.ARG,
        ],
    )
    def list(self):
        domain_id = self.app.pargs.domain_id
        response = self.app.client.get_certificates(
            domain_id=domain_id,
            qs={"page": getattr(self.app.pargs, PaginationEnum.KEY)}
        )
        rows = response.data.get('results', [])
        for row in rows:
            row['hostname'] = row['domain']['hostname']
            row['extra'] = json.dumps(row['extra_data'], indent=4)
        self.app.render(data=response.data,
                        rows=response.data.get('results', []),
                        headers={'pk': 'ID',
                                 'fqdn': 'FQDN',
                                 'hostname': 'Hostname',
                                 'status': 'Status',
                                 'expire_date': 'Expire Date',
                                 'extra': 'Extra Data'},
                        is_succeed=response.is_succeed)

    @ex(
        help='Certificate Create Command',
        arguments=[
            (['domain_id'], {
                'help': 'Domain ID',
                'action': 'store',
            }),
            (['fqdn'], {
                'help': 'FQDN',
                'action': 'store',
            })
        ]
    )
    def create(self):
        data = {
            'fqdn': self.app.pargs.fqdn,
        }
        domain_id = self.app.pargs.domain_id
        response = self.app.client.create_certificate(domain_id=domain_id,
                                                      data=data)
        if response.is_succeed:
            self.app.render({}, renderer_type='text',
                            custom_text="Certificate has been created.")
        else:
            custom_text = response.data.get('non_field_errors')
            if not custom_text:
                custom_text = response.data.get('fqdn')[0]
            self.app.render({}, renderer_type='text', custom_text=custom_text)



from cement import Controller, ex
from cement.utils.version import get_version_banner

from . import PaginationEnum
from ..core.version import get_version

VERSION_BANNER = """
CLI for Akinon Cloud Commerce %s
%s
""" % (get_version(), get_version_banner())


class PublicKeys(Controller):
    class Meta:
        label = 'publickey'
        stacked_type = 'nested'
        stacked_on = 'base'
        description = 'this is the public key controller namespace'

    @ex(
        help='Public Key List Command',
        arguments=[
            PaginationEnum.ARG
        ],
    )
    def list(self):
        response = self.app.client.get_public_keys(
            qs={"page": getattr(self.app.pargs, PaginationEnum.KEY)}
        )
        self.app.render(data=response.data, rows=response.data.get('results', []),
                        headers={'pk': 'ID', 'public_key': 'Public Key'},
                        is_succeed=response.is_succeed)

    @ex(
        help='Public Key Create Command',
        arguments=[
            (['label'], {
                'help': 'Label',
                'action': 'store',
            }),
            (['key'], {
                'help': 'Public Key',
                'action': 'store',
            }),
        ]
    )
    def create(self):
        data = {
            'public_key': self.app.pargs.key,
            'label': self.app.pargs.label
        }
        response = self.app.client.create_public_key(data)
        if response.is_succeed:
            self.app.render({}, renderer_type='text', custom_text="Public Key added successfully.")
        else:
            self.app.render({}, renderer_type='text', custom_text=response.data.get('non_field_errors') or response.data.get('public_key'))

    @ex(
        help='Public Key Remove Command',
        arguments=[
            (['id'], {
                'help': 'Public Key ID',
                'action': 'store',
            }),
        ]
    )
    def remove(self):
        response = self.app.client.remove_public_key(self.app.pargs.id)
        if response.is_succeed:
            self.app.render({}, renderer_type='text', custom_text="Public Key removed successfully.")
        else:
            self.app.render({}, renderer_type='text',
                            custom_text=response.data.get('non_field_errors') or response.data)



import getpass

from cement import Controller, ex
from cement.utils.version import get_version_banner

from akinoncli.core.exc import AkinonCLIError
from ..core.version import get_version


VERSION_BANNER = """
CLI for Akinon Cloud Commerce %s
%s
""" % (get_version(), get_version_banner())


# class Users(Controller):
#     class Meta:
#         label = 'user'
#         stacked_type = 'nested'
#         stacked_on = 'base'
#         description = 'this is the users controller namespace'
#
#     @ex(
#         help='User List Command',
#         arguments=[
#             (['-r', '--role'], {
#                 'help': 'Filter by role id',
#                 'action': 'store',
#                 'dest': 'role'
#             })
#         ],
#     )
#     def list(self):
#         qs = {}
#         if self.app.pargs.role:
#             qs['role'] = self.app.pargs.role
#         response = self.app.client.get_users(qs)
#         rows = response.data.get('results', [])
#         for row in rows:
#             row['role_name'] = row['role']['name']
#             row['role_id'] = row['role']['pk']
#         self.app.render(data=response.data, rows=rows,
#                         headers={'pk': 'ID', 'first_name': 'First Name', 'last_name': 'Last Name',
#                                  'email': 'Email', 'role_name': 'Role'},
#                         is_succeed=response.is_succeed)
#
#     @ex(
#         help='User Create Command',
#         arguments=[]
#     )
#     def create(self):
#         first_name = input('First Name:')
#         last_name = input('Last Name:')
#         role = input('Role ID:')
#         email = input('Email:')
#         password = getpass.getpass(prompt='Password:')
#         password2 = getpass.getpass(prompt='Password Confirm:')
#         if password != password2:
#             raise AkinonCLIError("Passwords did not match")
#         data = {
#             'first_name': first_name,
#             'last_name': last_name,
#             'email': email,
#             'password': password,
#             'role': role
#         }
#         response = self.app.client.create_user(data)
#         if response.is_succeed:
#             self.app.render({}, is_text=True, custom_text="User added successfully.")
#         else:
#             errors = ''
#             for key, value in response.data.items():
#                 errors += f'{key}: {value[0]}\n'
#             self.app.render({}, is_text=True, custom_text=errors)
#
#
# class Roles(Controller):
#     class Meta:
#         label = 'role'
#         stacked_type = 'nested'
#         stacked_on = 'base'
#         description = 'this is the roles controller namespace'
#
#     @ex(
#         help='Role List Command',
#         arguments=[],
#     )
#     def list(self):
#         response = self.app.client.get_roles()
#         self.app.render(data=response.data, rows=response.data.get('results', []),
#                         headers={'pk': 'ID', 'name': 'Name'},
#                         is_succeed=response.is_succeed)
#
#     @ex(
#         help='Role Create Command',
#         arguments=[
#             (['name'], {
#                 'help': 'Role Name',
#                 'action': 'store'
#             })
#         ],
#     )
#     def create(self):
#         data = {
#             'name': self.app.pargs.name
#         }
#         response = self.app.client.create_role(data)
#         if response.is_succeed:
#             self.app.render({}, is_text=True, custom_text="Role created successfully.")
#         else:
#             self.app.render({}, is_text=True, custom_text=response.data.get('non_field_errors') or response.data.get('name'))




from cement import Controller, ex
from cement.utils.version import get_version_banner

from . import PaginationEnum
from ..core.version import get_version


VERSION_BANNER = """
CLI for Akinon Cloud Commerce %s
%s
""" % (get_version(), get_version_banner())


class Applications(Controller):
    class Meta:
        label = 'application'
        stacked_type = 'nested'
        stacked_on = 'base'
        description = 'this is the application controller namespace'

    @ex(
        help='Application List Command',
        arguments=[
            PaginationEnum.ARG
        ],
    )
    def list(self):
        response = self.app.client.get_applications(
            qs={"page": getattr(self.app.pargs, PaginationEnum.KEY)}
        )
        results = []
        for result in response.data.get("results", []):
            application_type_data = result["application_type"]
            result['application_type_slug'] = application_type_data["slug"] if application_type_data else ""
            results.append(result)
        self.app.render(data=response.data, rows=results,
                        headers={'pk': 'ID', 'slug': 'Slug', 'name': 'Name',
                                 'is_private': 'Is Private', 'status': 'Status',
                                 'clone_url': 'Clone URL',
                                 'application_type_slug': 'Application Type Slug'},
                        is_succeed=response.is_succeed)

    @ex(
        help='Application Create Command',
        arguments=[
            (['name'], {
                'help': 'Application name',
                'action': 'store',
            }),
            (['slug'], {
                'help': 'Application slug',
                'action': 'store',
            }),
            (['application_type_id'], {
                'help': 'Application type id',
                'action': 'store',
            }),
        ]
    )
    def create(self):
        data = {
            'name': self.app.pargs.name,
            'slug': self.app.pargs.slug,
            'application_type': self.app.pargs.application_type_id,
            'is_private': True,
        }
        response = self.app.client.create_application(data)
        if response.is_succeed:
            self.app.render({}, renderer_type='text', custom_text="App has been created.")
        else:
            self.app.render({}, renderer_type='text', custom_text=response.data['slug'][0])

    @ex(
        help='Application Update Command',
        arguments=[
            (['id'], {
                'help': 'Application ID',
                'action': 'store',
            }),
            (['name'], {
                'help': 'Application Name',
                'action': 'store',
            }),
            (['slug'], {
                'help': 'Application slug',
                'action': 'store',
            }),
        ]
    )
    def update(self):
        app_id = self.app.pargs.id
        data = {
            'name': self.app.pargs.name,
            'slug': self.app.pargs.slug,
        }
        response = self.app.client.update_application(app_id, data)
        if response.is_succeed:
            self.app.render({}, renderer_type='text', custom_text="App has been updated.")
        else:
            self.app.render({}, renderer_type='text', custom_text=response.data)

    @ex(
        help='Application Build Command',
        arguments=[
            (['id'], {
                'help': 'Application ID',
                'action': 'store',
            }),
            (['tag'], {
                'help': 'App Version',
                'action': 'store',
            }),
            (['--note'], {
                'help': 'Version Note',
                'action': 'store'
            })
        ]
    )
    def build(self):
        app_id = self.app.pargs.id
        data = {
            'tag': self.app.pargs.tag,
        }
        if self.app.pargs.note is not None:
            data['patch_notes'] = self.app.pargs.note
        response = self.app.client.build_application(app_id, data)
        if response.is_succeed:
            self.app.render({}, renderer_type='text', custom_text="Build process has been started.")
        else:
            self.app.render({}, renderer_type='text', custom_text=response.data["non_field_errors"])

    @ex(
        help='Application Version List Command',
        arguments=[
            (['id'], {
                'help': 'Application ID',
                'action': 'store'
            })
        ],
    )
    def versions(self):
        app_id = self.app.pargs.id
        response = self.app.client.get_app_versions(app_id)
        self.app.render(data=response.data, rows=response.data.get('results', []),
                        headers={'pk': 'ID', 'app': 'App', 'tag': 'Tag',
                                 'status': 'Status', 'created_date': 'CreatedDate'},
                        is_succeed=response.is_succeed)

    @ex(
        help='Application Version Log List Command',
        arguments=[
            (['id'], {
                'help': 'Application ID',
                'action': 'store'
            }),
            (['version_id'], {'help': 'Version ID', 'action': 'store'})
        ],
    )
    def version_logs(self):
        app_id = self.app.pargs.id
        version_id = self.app.pargs.version_id
        response = self.app.client.get_app_version_logs(app_id=app_id, version_id=version_id)
        self.app.render(
            data=response.data, rows=response.data, renderer_type="pipeline_log", is_succeed=response.is_succeed
        )


class ApplicationTypes(Controller):
    class Meta:
        label = 'applicationtype'
        stacked_type = 'nested'
        stacked_on = 'base'
        description = 'this is the application type controller namespace'

    @ex(
        help='Application Type List Command',
        arguments=[
            PaginationEnum.ARG
        ],
    )
    def list(self):
        qs = {"is_visible_on_app_creation": True, "page": getattr(self.app.pargs, PaginationEnum.KEY)}
        response = self.app.client.get_application_types(qs=qs)
        self.app.render(data=response.data, rows=response.data.get('results', []),
                        headers={'pk': 'ID', 'slug': 'Slug', 'name': 'Name',
                                 'created_date': 'Created Date',
                                 'update_date': 'Updated Date'},
                        is_succeed=response.is_succeed)



import os
from datetime import datetime, timezone, timedelta

from cement import Controller, ex
from cement.utils.version import get_version_banner

from ..core.exc import AkinonCLIError, AkinonCLIWarning
from . import PaginationEnum
from ..core.version import get_version

VERSION_BANNER = """
CLI for Akinon Cloud Commerce %s
%s
""" % (get_version(), get_version_banner())


class Projects(Controller):
    class Meta:
        label = 'project'
        stacked_type = 'nested'
        stacked_on = 'base'
        description = 'this is the project controller namespace'

    @ex(
        help='Project List Command',
        arguments=[
            PaginationEnum.ARG
        ],
    )
    def list(self):
        response = self.app.client.get_projects(
            qs={"page": getattr(self.app.pargs, PaginationEnum.KEY)}
        )
        self.app.render(data=response.data, rows=response.data.get('results', []),
                        headers={'pk': 'ID', 'slug': 'Slug', 'name': 'Name'},
                        is_succeed=response.is_succeed)

    @ex(
        help='Project Create Command',
        arguments=[
            (['name'], {
                'help': 'Project name',
                'action': 'store',
            }),
            (['slug'], {
                'help': 'Project slug',
                'action': 'store',
            }),
            (['--mainapp-required'], {
                'help': 'if main app is required. Choose main app required as true.',
                'action': 'store_true',
                'dest': 'is_main_app_required'
            }),
            (['--mainapp-initialize'], {
                'help': 'Allow you to initialize main app automatically.',
                'action': 'store_true',
                'dest': 'initialize_main_app'
            }),
        ]
    )
    def create(self):
        data = {
            'name': self.app.pargs.name,
            'slug': self.app.pargs.slug,
            'is_main_app_required': self.app.pargs.is_main_app_required,
            'initialize_default_app_on_creation': self.app.pargs.initialize_main_app,
        }
        response = self.app.client.create_project(data)
        if response.is_succeed:
            self.app.render({}, renderer_type='text', custom_text="Project has been created.")
        else:
            self.app.render({}, renderer_type='text', custom_text=response.data['slug'][0])

    @ex(
        help='Project Update Command',
        arguments=[
            (['id'], {
                'help': 'Project ID',
                'action': 'store',
            }),
            (['name'], {
                'help': 'New project name',
                'action': 'store',
            }),
        ]
    )
    def update(self):
        p_id = self.app.pargs.id
        data = {
            'name': self.app.pargs.name,
        }
        response = self.app.client.update_project(p_id, data)
        if response.is_succeed:
            self.app.render({}, renderer_type='text', custom_text="Project has been updated.")
        else:
            self.app.render({}, renderer_type='text', custom_text=response.data)


class ProjectApps(Controller):
    class Meta:
        label = 'projectapp'
        stacked_type = 'nested'
        stacked_on = 'base'
        description = 'this is the project app controller namespace'

    @ex(
        help='Project App List Command',
        arguments=[
            (['project_id'], {
                'help': 'Project ID',
                'action': 'store',
            }),
            PaginationEnum.ARG,
        ],
    )
    def list(self):
        p_id = self.app.pargs.project_id
        response = self.app.client.get_project_apps(
            p_id=p_id,
            qs={"page": getattr(self.app.pargs, PaginationEnum.KEY)}
        )
        rows = response.data.get('results', [])
        for row in rows:
            row['app'] = row['app']['name']
            row['project'] = row['project']['name']
            row['env'] = ''
            for key, value in row['custom_env'].items():
                row['env'] += f'{key}={value}\n'
        self.app.render(data=response.data, rows=response.data.get('results', []),
                        headers={'pk': 'ID', 'project': 'Project', 'app': 'App',
                                 'url': 'URL', 'created_date': 'Created Date',
                                 'env': 'ENV Variables'},
                        is_succeed=response.is_succeed)

    @ex(
        help='Project App Add or Update Environment Value Command',
        arguments=[
            (['project_id'], {
                'help': 'Project ID',
                'action': 'store',
            }),
            (['project_app_id'], {
                'help': 'Project App ID',
                'action': 'store',
            }),
            (['env_variables'],
             {'help':  'Environment Variables (KEY=VALUE)', 'action': 'store', 'nargs': '+'}),
            (['--deploy'], {'help': "Redeploy the current version to activate environment variable changes.",
                               'action': 'store_true', 'dest': 'deploy', 'default': False}),
        ]
    )
    def add_env(self):
        p_id = self.app.pargs.project_id
        pa_id = self.app.pargs.project_app_id
        new_env_variables = dict([tuple(env.split('=', maxsplit=1)) for env in self.app.pargs.env_variables])
        response = self.app.client.update_project_app_custom_env(p_id, pa_id, data=new_env_variables)
        project_app = self.app.client.get_project_app(p_id, pa_id)
        if response.is_succeed:
            if self.app.pargs.deploy:
                current_deployment_tag = {'tag': project_app.data['current_deployment']['version']}
                self.app.client.deploy_project_app(p_id, pa_id, data=current_deployment_tag)

            row = response.data
            row['env'] = ''
            row['pk'] = project_app.data['pk']
            row['created_date'] = project_app.data['created_date']
            row['app'] = project_app.data['app']['name']
            row['project'] = project_app.data['project']['name']
            for key, value in row['custom_env'].items():
                row['env'] += f'{key}={value}\n'

            self.app.render(data=response.data, rows=[row],
                            headers={'pk': 'ID', 'project': 'Project', 'app': 'App',
                                     'created_date': 'Created Date', 'env': 'ENV Variables'},
                            is_succeed=response.is_succeed)
        else:
            self.app.render({}, renderer_type='text', custom_text=response.data)

    @ex(
        help='Project App Remove Environment Value Command',
        arguments=[
            (['project_id'], {
                'help': 'Project ID',
                'action': 'store',
            }),
            (['project_app_id'], {
                'help': 'Project App ID',
                'action': 'store',
            }),
            (['env_keys'], {
                'help': 'Keys',
                'action': 'store',
                'nargs': '+'
            }),
            (['--deploy'], {'help': "Redeploy the current version to activate environment variable changes.",
                               'action': 'store_true', 'dest': 'deploy', 'default': False}),
        ]
    )
    def remove_env(self):
        p_id = self.app.pargs.project_id
        pa_id = self.app.pargs.project_app_id
        env_keys_to_delete = self.app.pargs.env_keys
        response = self.app.client.delete_project_app_custom_env(p_id, pa_id, data=env_keys_to_delete)
        project_app = self.app.client.get_project_app(p_id, pa_id)

        if response.is_succeed:
            if self.app.pargs.deploy:
                current_deployment_tag = {'tag': project_app.data['current_deployment']['version']}
                self.app.client.deploy_project_app(p_id, pa_id, data=current_deployment_tag)

            row = response.data
            row['env'] = ''
            row['pk'] = project_app.data['pk']
            row['created_date'] = project_app.data['created_date']
            row['app'] = project_app.data['app']['name']
            row['project'] = project_app.data['project']['name']

            for key, value in row['custom_env'].items():
                row['env'] += f'{key}={value}\n'

            self.app.render(data=response.data, rows=[row],
                            headers={'pk': 'ID', 'project': 'Project', 'app': 'App',
                                     'created_date': 'Created Date', 'env': 'ENV Variables'},
                            is_succeed=response.is_succeed)
        else:
            self.app.render({}, renderer_type='text', custom_text=response.data)

    @ex(
        help='Project App Add Command',
        arguments=[
            (['project_id'], {
                'help': 'Project ID',
                'action': 'store',
            }),
            (['app_id'], {
                'help': 'App ID',
                'action': 'store',
            }),
        ],
    )
    def add(self):
        p_id = self.app.pargs.project_id
        app_id = self.app.pargs.app_id
        data = {
            'app': app_id
        }
        response = self.app.client.create_project_app(p_id, data)
        if response.is_succeed:
            self.app.render({}, renderer_type='text', custom_text="App has been added to the project.")
        else:
            self.app.render({}, renderer_type='text', custom_text=response.data)

    @ex(
        help='Project App Deploy Command',
        arguments=[
            (['project_id'], {
                'help': 'Project ID',
                'action': 'store',
            }),
            (['project_app_id'], {
                'help': 'Project App ID',
                'action': 'store',
            }),
            (['tag'], {
                'help': 'Tag',
                'action': 'store',
            }),
        ],
    )
    def deploy(self):
        p_id = self.app.pargs.project_id
        pa_id = self.app.pargs.project_app_id
        data = {
            'tag': self.app.pargs.tag
        }
        response = self.app.client.deploy_project_app(p_id, pa_id, data=data)

        if response.is_succeed:
            self.app.render({}, renderer_type='text', custom_text="ProjectApp deployment has been started.")
        else:
            self.app.render({}, renderer_type='text', custom_text=response.data['non_field_errors'])

    @ex(
        help='Multiple Project App Deploy Command',
        arguments=[
            (['app_id'], {
                'help': 'Application ID',
                'action': 'store',
            }),
            (['tag'], {
                'help': 'App Version',
                'action': 'store',
            }),
            (['--project-apps'], {
                'help': 'Project App IDs',
                'action': 'store',
                'nargs': '+',
                'default': [],
            }),
            (['--deploy-all'], {
                'help': "Deploys all project apps with given app_id and tag.",
                'action': 'store_true',
                'dest': 'deploy_all',
                'default': False,
            }),
        ]
    )
    def deploy_multiple(self):
        data = {
            'tag': self.app.pargs.tag,
            'project_apps': self.app.pargs.project_apps or [],
            'deploy_all': self.app.pargs.deploy_all,
        }
        response = self.app.client.deploy_multiple_project_apps(self.app.pargs.app_id, data)
        if response.is_succeed:
            self.app.render({}, renderer_type='text', custom_text="ProjectApp deployments have been started.")
        else:
            self.app.render({}, renderer_type='text', custom_text=response.data["non_field_errors"])

    @ex(
        help='Project App Deployment List Command',
        arguments=[
            (['project_id'], {
                'help': 'Project ID',
                'action': 'store',
            }),
            (['project_app_id'], {
                'help': 'Project App ID',
                'action': 'store',
            })
        ],
    )
    def deployments(self):
        p_id = self.app.pargs.project_id
        pa_id = self.app.pargs.project_app_id
        response = self.app.client.get_project_app_deployments(p_id, pa_id)
        self.app.render(data=response.data, rows=response.data.get('results', []),
                        headers={'pk': 'ID', 'version': 'Version', 'status': 'Status',
                                 'created_date': 'Created Date'},
                        is_succeed=response.is_succeed)

    @ex(
        help='Project App Deployment Log List Command',
        arguments=[
            (['project_id'], {
                'help': 'Project ID',
                'action': 'store',
            }),
            (['project_app_id'], {
                'help': 'Project App ID',
                'action': 'store',
            }),
            (['deployment_id'], {'help': 'Deployment ID', 'action': 'store'})
        ],
    )
    def deployment_logs(self):
        p_id = self.app.pargs.project_id
        pa_id = self.app.pargs.project_app_id
        deployment_id = self.app.pargs.deployment_id
        response = self.app.client.get_project_app_deployment_logs(p_id, pa_id, deployment_id=deployment_id)
        self.app.render(
            data=response.data, rows=response.data, renderer_type="pipeline_log", is_succeed=response.is_succeed
        )

    @ex(
        help='Attach Certificate to Project App',
        arguments=[
            (['project_id'], {
                'help': 'Project ID',
                'action': 'store',
            }),
            (['project_app_id'], {
                'help': 'Project App ID',
                'action': 'store',
            }),
            (['certificate_id'], {
                'help': 'Certificate ID',
                'action': 'store',
            }),
        ],
    )
    def attach_certificate(self):
        p_id = self.app.pargs.project_id
        pa_id = self.app.pargs.project_app_id
        data = {
            'certificate': self.app.pargs.certificate_id
        }
        response = self.app.client.update_project_app(p_id, pa_id, data=data)

        if response.is_succeed:
            self.app.render({}, renderer_type='text',
                            custom_text="Certificate has been attached"
                                        " to project app.")
        else:
            self.app.render({}, renderer_type='text',
                            custom_text=response.data['non_field_errors'])

    @ex(
        help='Detach Certificate to Project App',
        arguments=[
            (['project_id'], {
                'help': 'Project ID',
                'action': 'store',
            }),
            (['project_app_id'], {
                'help': 'Project App ID',
                'action': 'store',
            })
        ],
    )
    def detach_certificate(self):
        p_id = self.app.pargs.project_id
        pa_id = self.app.pargs.project_app_id
        data = {
            'certificate': None
        }
        response = self.app.client.update_project_app(p_id, pa_id, data=data)

        if response.is_succeed:
            self.app.render({}, renderer_type='text',
                            custom_text="Certificate has been detached"
                                        " from project app.")
        else:
            self.app.render({}, renderer_type='text',
                            custom_text=response.data['non_field_errors'])

    @ex(
        help='Get Project App Logs',
        arguments=[
            (['project_id'], {'help': 'Project ID', 'action': 'store', }),
            (['project_app_id'], {'help': 'Project App ID', 'action': 'store'}),
            (['-o', '--offset'], {'help': 'Offset (second)', 'action': 'store', 'dest': "offset", 'default': 10}),
            (['-l', '--limit'], {'help': 'Limit (second)', 'action': 'store', 'dest': "limit", 'default': 5}),
            (['-p', '--process'], {'help': 'Process type', 'action': 'store', 'dest': "process_type", 'default': ''}),
        ],
    )
    def logs(self):
        offset = int(self.app.pargs.offset)
        limit = int(self.app.pargs.limit)
        process_type = self.app.pargs.process_type or []
        if process_type:
            process_type = [process_type]
        query_end_datetime = datetime.now(timezone.utc) - timedelta(seconds=offset)
        query_start_datetime = query_end_datetime - timedelta(seconds=limit)
        project_id = self.app.pargs.project_id
        project_app_id = self.app.pargs.project_app_id
        response = self.app.client.get_project_app_logs(
            project_id=project_id,
            project_app_id=project_app_id,
            start_datetime=query_start_datetime,
            end_datetime=query_end_datetime,
            process_type=process_type
        )
        rows = response.data.get('items', [])
        for row in rows:
            if 'message' not in row:
                row['message'] = ""
        self.app.render(
            renderer_type="realtime_log",
            data=response.data,
            rows=rows,
            is_succeed=response.is_succeed
        )

    @ex(
        help='Export Project App Logs',
        arguments=[
            (['project_id'], {'help': 'Project ID', 'action': 'store', }),
            (['project_app_id'], {'help': 'Project App ID', 'action': 'store'}),
            (['-d', '--dates'], {'help': 'Dates (YYYY-MM-DD)', 'action': 'store', 'dest': 'dates', 'default': ''}),
            (['-p', '--process'], {'help': 'Process type', 'action': 'store', 'default': ''}),
            (['-s', '--start_date'], {'help': 'Start date (YYYY-MM-DD HH:MM)', 'action': 'store', 'dest': 'start_date'}),
            (['-e', '--end_date'], {'help': 'End date (YYYY-MM-DD HH:MM)', 'action': 'store', 'dest': 'end_date'}),
        ]
    )
    def export_logs(self):
        project_id = self.app.pargs.project_id
        project_app_id = self.app.pargs.project_app_id
        dates = self.app.pargs.dates
        applications = self.app.pargs.process
        start_date = self.app.pargs.start_date
        end_date = self.app.pargs.end_date

        if dates and (start_date or end_date):
            raise AkinonCLIError("--dates and (--start_date or --end_date) filters cannot be used together.")

        if start_date and end_date:
            try:
                datetime.strptime(start_date, "%Y-%m-%d %H:%M")
                datetime.strptime(end_date, "%Y-%m-%d %H:%M")
            except:
                raise AkinonCLIError("start_date and end_date must be in the format 'YYYY-MM-DD HH:MM'")

        try:
            response = self.app.client.export_project_app_logs(
                project_id=project_id,
                project_app_id=project_app_id,
                dates=dates,
                applications=applications,
                start_date=start_date,
                end_date=end_date
            )
        except AkinonCLIError as e:
            if e.response.status_code == 406:
                raise AkinonCLIWarning(e.response.json()['non_field_errors'])
            else:
                raise

        self.app.render({}, custom_text=response.data['message'], renderer_type='text')


class Addons(Controller):
    class Meta:
        label = 'addon'
        stacked_type = 'nested'
        stacked_on = 'base'
        description = 'this is the addon controller namespace'

    @ex(
        help='Addon List Command',
        arguments=[
            (['project_id'], {'help': 'Project ID', 'action': 'store', }),
            (['project_app_id'], {'help': 'Project App ID', 'action': 'store'}),
        ],
    )
    def list(self):
        response = self.app.client.get_addons(
            project_id=int(self.app.pargs.project_id),
            project_app_id=int(self.app.pargs.project_app_id)
        )
        self.app.render(
            data=response.data,
            rows=response.data.get('results', []),
            headers={
                "pk": "ID",
                "addon_type_relation": "Addon Type Relation",
                "project_app": "Project App ID",
                "is_alive": "Is Alive",
                "config": "Config",
                "remote_url": "Remote URL",
                "status": "Status",
                "is_provisioned": "Is Provisioned"
            },
            is_succeed=response.is_succeed
        )




from cement import Controller, ex
from cement.utils.version import get_version_banner

from ..core.version import get_version


VERSION_BANNER = """
CLI for Akinon Cloud Commerce %s
%s
""" % (get_version(), get_version_banner())


class Base(Controller):
    class Meta:
        label = 'base-url'
        stacked_type = 'nested'
        stacked_on = 'base'
        description = 'this is the base controller namespace'

    @ex(
        help='Set URLCommand',
        arguments=[
            (['url'], {
                'help': 'Base URL',
                'action': 'store',
            }),
        ],
    )
    def set(self):
        table = self.app.db.table('urls')
        if len(table) != 0:
            table.remove(doc_ids=[1])
        table.insert({'base_url': self._format_base_url(self.app.pargs.url)})

    def _format_base_url(self, base_url: str):
        if not base_url.endswith("/"):
            base_url += "/"
        if not base_url.endswith(f"/{self._api_suffix}"):
            base_url += self._api_suffix
        return base_url

    @property
    def _api_suffix(self) -> str:
        return "api/v1/"



from typing import Optional

import requests


class BaseAkinonCLIException(Exception):
    def __init__(self, message: str, *, response: Optional[requests.Response] = None):
        super().__init__(message, response)
        self.message = message
        self.response = response


class AkinonCLIError(BaseAkinonCLIException):
    """Generic errors."""
    ...


class AkinonCLIWarning(BaseAkinonCLIException):
    """Generic warnings."""
    ...



import os

from akinoncli.controllers.clusters import KubeMetricMonitor
from tinydb import TinyDB
from cement.utils import fs
from cement import App, TestApp, init_defaults
from cement.core.exc import CaughtSignal

from akinoncli.controllers.account import Domains, Certificates
from akinoncli.ext.output_renderer import AkinonOutputHandler
from akinoncli.client.client import AkinonCloudClient
from akinoncli.core.exc import AkinonCLIError, AkinonCLIWarning

from akinoncli.controllers.base import Base
from akinoncli.controllers.auth import Auth
# from .controllers.clusters import Clusters
from akinoncli.controllers.projects import Projects, ProjectApps, Addons
from akinoncli.controllers.applications import Applications, ApplicationTypes
# from .controllers.applications import ApplicationUsers
from akinoncli.controllers.public_keys import PublicKeys

# from .controllers.users import Users, Roles

# configuration defaults
CONFIG = init_defaults('akinoncli')
CONFIG['akinoncli']['db_file'] = '~/.akinoncli/db.json'


def extend_tinydb(app):
    db_file = app.config.get('akinoncli', 'db_file')
    db_file = fs.abspath(db_file)
    # ensure our parent directory exists
    db_dir = os.path.dirname(db_file)
    if not os.path.exists(db_dir):
        os.makedirs(db_dir)

    app.extend('db', TinyDB(db_file))


def extend_client(app):
    urls_table = app.db.table('urls')
    base_url = 'http://localhost:8000/api/v1/'
    if len(urls_table) > 0:
        base_url = urls_table.get(doc_id=1).get('base_url')
    user = app.db.get(doc_id=1)
    token = None
    if user:
        token = user.get('token')
    app.extend('client', AkinonCloudClient(base_url, token))


class AkinonCLI(App):
    """Akinon CLI primary application."""

    class Meta:
        label = 'akinoncli'

        # configuration defaults
        config_defaults = CONFIG

        # call sys.exit() on close
        exit_on_close = True

        # load additional framework extensions
        extensions = [
            'yaml',
            'colorlog',
        ]

        # configuration handler
        config_handler = 'yaml'

        # configuration file suffix
        config_file_suffix = '.yml'

        # set the log handler
        log_handler = 'colorlog'

        # set the output handler
        output_handler = 'akinon_output_handler'

        # register handlers
        handlers = [
            AkinonOutputHandler,
            Base,
            Auth,
            # Clusters,
            Projects,
            ProjectApps,
            Applications,
            # ApplicationUsers,
            ApplicationTypes,
            PublicKeys,
            # Users,
            # Roles,
            Domains,
            Certificates,
            Addons,
            KubeMetricMonitor,
        ]

        hooks = [
            ('post_setup', extend_tinydb),
            ('post_setup', extend_client),
        ]


class AkinonCLITest(TestApp, AkinonCLI):
    """A sub-class of AkinonCLI that is better suited for testing."""

    class Meta:
        label = 'akinoncli'


def main():
    with AkinonCLI() as app:
        try:
            app.run()

        except AssertionError as e:
            print(f'AssertionError > {e.args[0]}')
            app.exit_code = 1

            if app.debug is True:
                import traceback
                traceback.print_exc()

        except AkinonCLIError as e:
            app.log.error(f'{e.message}')
            app.exit_code = 1

            if e.response is not None:
                app.log.error(e.response.text)
            if app.debug is True:
                import traceback
                traceback.print_exc()
        except AkinonCLIWarning as e:
            app.log.warning(f'{e.message}')
        except CaughtSignal as e:
            # Default Cement signals are SIGINT and SIGTERM, exit 0 (non-error)
            print(f'\n{e}')
            app.exit_code = 0


if __name__ == '__main__':
    main()



import getpass

from cement import Controller, ex
from cement.utils.version import get_version_banner

from akinoncli.core.exc import AkinonCLIError
from ..core.version import get_version

VERSION_BANNER = """
CLI for Akinon Cloud Commerce %s
%s
""" % (
    get_version(),
    get_version_banner(),
)


class Auth(Controller):
    class Meta:
        label = 'auth'
        stacked_type = 'embedded'
        stacked_on = 'base'
        description = 'this is the auth controller namespace'

    @ex(help='Login Command')
    def login(self):
        if self.app.client.token:
            raise AkinonCLIError(
                "You have already logged in. If you would like to "
                "change your credentials please run 'logout' command"
            )

        email = input('Email:')
        password = getpass.getpass(prompt='Password:')
        data = {
            'email': email,
            'password': password,
        }

        response = self.app.client.login(data)
        if response.is_succeed:
            self.app.db.insert(
                {
                    "token": response.data["token"],
                    "account": response.data["account"],
                }
            )
            self.app.render(
                {},
                renderer_type='text',
                custom_text=f"Welcome {response.data['first_name']} {response.data['last_name']}",
            )
        else:
            self.app.render({}, renderer_type='text', custom_text=response.data["non_field_errors"][0])

    @ex(
        help='Logout Command',
        arguments=[],
    )
    def logout(self):
        if self.app.client.token:
            self.app.db.remove(doc_ids=[1])



import requests

from akinoncli.core.exc import AkinonCLIError


class AkinonResponse:
    def __init__(self, is_succeed, status_code, data=None):
        data = data or {}
        self.is_succeed = is_succeed
        self.status_code = status_code
        self.data = data


class AkinonCloudClient:
    URLS = {
        "base": "",
        "apps": "applications/",
        "app_detail": "applications/{app_id}/",
        "app_build": "applications/{app_id}/build/",
        "app_versions": "applications/{app_id}/versions/",
        "app_version_logs": "applications/{app_id}/versions/{v_id}/logs",
        "app_version_detail": "applications/{app_id}/versions/{v_id}/",
        "application_types": "applications/application_types/",
        "application_types_admin": "admin/application_types/",
        "application_types_admin_detail": "admin/application_types/{application_type_id}/",
        "app_users": "applications/{app_id}/users/",
        "clusters": "clusters/",
        "login": "users/login/",
        "projects": "projects/",
        "project_detail": "projects/{p_id}/",
        "project_apps": "projects/{p_id}/project_apps/",
        "project_app_detail": "projects/{p_id}/project_apps/{pa_id}/",
        "project_app_custom_env": "projects/{p_id}/project_apps/{pa_id}/custom_env/",
        "project_app_deploy": "projects/{p_id}/project_apps/{pa_id}/deploy/",
        "project_app_deploy_multiple": "applications/{app_id}/deploy_multiple_project_apps/",
        "project_app_deployments": "projects/{p_id}/project_apps/{pa_id}/deployments/",
        "project_app_deployment_log": "projects/{p_id}/project_apps/{pa_id}/deployments/{deployment_id}/logs",
        "project_apps_logs": "projects/{project_id}/project_apps/{project_app_id}/logs/",
        "project_apps_export_logs": "projects/{project_id}/project_apps/{project_app_id}/logs/export",
        "public_keys": "users/public_keys/",
        "public_key_detail": "users/public_keys/{pk_id}/",
        "register": "users/register/",
        "users": "users/",
        "roles": "users/roles/",
        "domains": "accounts/domains/",
        "certificates": "accounts/domains/{domain_id}/certificates/",
        "addons": "projects/{project_id}/project_apps/{project_app_id}/addons/",
        "kube_metric_monitor": "clusters/{cluster}/monitor/{namespace}/",
    }

    def __init__(self, base_url, token):
        self.token = token
        self.URLS['base'] = base_url

    def _send_request(self, method, url, params=None, data=None, headers=None, qs=None, response_format='json'):
        params = params or {}
        data = data or {}
        headers = headers or {}
        qs = qs or {}
        headers["Content-Type"] = "application/json"
        if self.token:
            headers["Authorization"] = f"Token {self.token}"
        url = f"{self.URLS['base']}{self.URLS[url]}".format(**params)
        response = requests.request(method=method, url=url, json=data,
                                    headers=headers, params=qs)
        if response.status_code >= 500:
            raise AkinonCLIError("An error has occurred on our side. We'll fix that.", response=response)
        elif response.status_code == 401:
            raise AkinonCLIError("Unauthenticated request, please log in with your credentials.", response=response)
        elif response.status_code == 403:
            raise AkinonCLIError("You do not have access to do this action. Please contact your account owner.",
                                 response=response)
        elif not response.ok:
            raise AkinonCLIError(
                f'API returned an unsuccessful status code. HTTP {response.status_code}: {response.reason}',
                response=response)

        data = response.json() if response_format == 'json' else response.content

        return AkinonResponse(is_succeed=response.ok,
                              status_code=response.status_code,
                              data=data)

    def login(self, data):
        return self._send_request("POST", "login", data=data)

    def get_clusters(self):
        return self._send_request("GET", "clusters")

    def get_projects(self, qs: dict):
        return self._send_request("GET", "projects", qs=qs)

    def create_project(self, data):
        return self._send_request("POST", "projects", data=data)

    def update_project(self, p_id, data):
        response = self._send_request("PATCH", "project_detail", params={"p_id": p_id},
                                      data=data)
        if response.status_code == 404:
            raise AkinonCLIError("Project could not found.")
        return response

    def get_applications(self, qs: dict):
        """
        return account's applications
        """
        return self._send_request("GET", "apps", params=qs)

    def create_application(self, data):
        return self._send_request("POST", "apps", data=data)

    def update_application(self, app_id, data):
        response = self._send_request("PATCH", "app_detail", params={"app_id": app_id},
                                      data=data)
        if response.status_code == 404:
            raise AkinonCLIError("Application could not found.")
        return response

    def build_application(self, app_id, data):
        response = self._send_request("POST", "app_build", params={"app_id": app_id},
                                      data=data)
        if response.status_code == 404:
            raise AkinonCLIError("Application could not found.")
        return response

    def get_app_versions(self, app_id):
        return self._send_request("GET", "app_versions", params={"app_id": app_id})

    def get_app_version_logs(self, app_id, version_id):
        return self._send_request("GET", "app_version_logs", params={"app_id": app_id, "v_id": version_id})

    def get_app_users(self, app_id):
        return self._send_request("GET", "app_users", params={"app_id": app_id})

    def create_app_user(self, app_id, data):
        return self._send_request("POST", "app_users", params={"app_id": app_id},
                                  data=data)

    def get_application_types(self, qs: dict):
        return self._send_request("GET", "application_types", qs=qs)

    def get_project_apps(self, p_id, qs: dict):
        return self._send_request("GET", "project_apps", params={"p_id": p_id}, qs=qs)

    def get_project_app(self, p_id, pa_id):
        return self._send_request("GET", "project_app_detail", params={"p_id": p_id,
                                                                       "pa_id": pa_id})

    def update_project_app(self, p_id, pa_id, data):
        response = self._send_request("PATCH", "project_app_detail", params={"p_id": p_id, "pa_id": pa_id},
                                      data=data)
        if response.status_code == 404:
            raise AkinonCLIError("Project App could not found.")
        return response

    def delete_project_app_custom_env(self, p_id, pa_id, data):
        response = self._send_request("POST", "project_app_custom_env", params={"p_id": p_id, "pa_id": pa_id},
                                      data=data)
        if response.status_code == 404:
            raise AkinonCLIError("Project App could not found.")
        return response

    def update_project_app_custom_env(self, p_id, pa_id, data):
        response = self._send_request("PUT", "project_app_custom_env", params={"p_id": p_id, "pa_id": pa_id}, data=data)
        if response.status_code == 404:
            raise AkinonCLIError("Project App could not found.")
        return response

    def get_project_app_logs(self, project_id, project_app_id, start_datetime, end_datetime, process_type):
        data = {
            "start_datetime": start_datetime.isoformat(),
            "end_datetime": end_datetime.isoformat(),
            "page": 1,
            "page_size": 10000,
            "applications": process_type
        }
        return self._send_request("POST", "project_apps_logs",
                                  params={"project_id": project_id, "project_app_id": project_app_id},
                                  data=data)

    def export_project_app_logs(self, project_id, project_app_id, start_date=None, end_date=None, dates=None,
                                applications=None):
        data = {}

        if start_date:
            data['start_date'] = start_date

        if end_date:
            data['end_date'] = end_date

        if dates:
            data['dates'] = dates.split(',')

        if applications:
            data['applications'] = applications.split(',')

        return self._send_request('POST', 'project_apps_export_logs',
                                  params={"project_id": project_id, "project_app_id": project_app_id},
                                  data=data)

    def create_project_app(self, p_id, data):
        return self._send_request("POST", "project_apps", params={"p_id": p_id},
                                  data=data)

    def deploy_project_app(self, p_id: int, pa_id: int, data: dict) -> AkinonResponse:
        response = self._send_request(
            "POST",
            "project_app_deploy",
            params={
                "p_id": p_id,
                "pa_id": pa_id,
            },
            data=data
        )
        if response.status_code == 404:
            raise AkinonCLIError("Project App could not be found.")
        return response

    def deploy_multiple_project_apps(self, app_id: int, data: dict) -> AkinonResponse:
        response = self._send_request(
            "POST",
            "project_app_deploy_multiple",
            params={
                "app_id": app_id,
            },
            data=data
        )
        if response.status_code == 404:
            raise AkinonCLIError("App could not be found.")
        return response

    def get_project_app_deployments(self, p_id, pa_id):
        return self._send_request("GET", "project_app_deployments", params={"p_id": p_id,
                                                                            "pa_id": pa_id})

    def get_project_app_deployment_logs(self, p_id, pa_id, deployment_id):
        return self._send_request("GET", "project_app_deployment_log", params={"p_id": p_id,
                                                                               "pa_id": pa_id,
                                                                               "deployment_id": deployment_id})

    def get_public_keys(self, qs: dict):
        return self._send_request("GET", "public_keys", qs=qs)

    def create_public_key(self, data):
        return self._send_request("POST", "public_keys", data=data)

    def remove_public_key(self, pk_id):
        return self._send_request("DELETE", "public_key_detail", params={"pk_id": pk_id})

    def get_users(self, qs=None):
        qs = qs or {}
        return self._send_request("GET", "users", qs=qs)

    def create_user(self, data):
        return self._send_request("POST", "users", data=data)

    def get_roles(self):
        return self._send_request("GET", "roles")

    def create_role(self, data):
        return self._send_request("POST", "roles", data=data)

    def get_domains(self, qs: dict):
        return self._send_request("GET", "domains", qs=qs)

    def create_domain(self, data):
        return self._send_request("POST", "domains", data=data)

    def get_certificates(self, domain_id, qs: dict):
        return self._send_request("GET", "certificates",
                                  params={"domain_id": domain_id}, qs=qs)

    def create_certificate(self, domain_id, data):
        return self._send_request("POST", "certificates", data=data,
                                  params={'domain_id': domain_id})

    def get_addons(self, project_id: int, project_app_id: int):
        return self._send_request(
            "GET",
            "addons",
            params={"project_id": project_id, "project_app_id": project_app_id},
        )

    def get_metrics(self, cluster: str, namespace: str, qs: dict):
        return self._send_request(
            "GET",
            "kube_metric_monitor",
            params={"cluster": cluster, "namespace": namespace},
            qs=qs,
        )



from setuptools import setup, find_packages
from akinoncli.core.version import get_version

__author__ = 'Akinon'
__license__ = 'MIT'
__maintainer__ = 'Akinon'
__email__ = 'dev@akinon.com'

VERSION = get_version()

with open('README.md', 'r', encoding='utf-8') as readme:
    long_description = readme.read()

setup(
    name='akinoncli',
    version=VERSION,
    description='CLI for Akinon Cloud Commerce',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=__author__,
    author_email=__email__,
    maintainer=__maintainer__,
    maintainer_email=__email__,
    url='https://bitbucket.org/akinonteam/akinon-cli/',
    license=__license__,
    install_requires=[
        "cement==3.0.4",
        "pyyaml==6.0",
        "colorlog==6.6.0",
        "tinydb==4.3.0",
        "requests==2.25.0",
        "rich==11.2.0"
    ],
    packages=find_packages(exclude=['ez_setup', 'tests*']),
    package_data={'akinoncli': ['templates/*']},
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Environment :: Console'
    ],
    include_package_data=True,
    entry_points="""
        [console_scripts]
        akinoncli = akinoncli.main:main
    """,
)



from cement import Controller, ex
from cement.utils.version import get_version_banner

from . import PaginationEnum
from ..core.version import get_version

VERSION_BANNER = """
CLI for Akinon Cloud Commerce %s
%s
""" % (get_version(), get_version_banner())


# class Clusters(Controller):
#     class Meta:
#         label = 'cluster'
#         stacked_type = 'nested'
#         stacked_on = 'base'
#         description = 'this is the cluster controller namespace'
#
#     @ex(
#         help='Cluster List Command',
#         arguments=[],
#     )
#     def list(self):
#         response = self.app.client.get_clusters()
#         self.app.render(data=response.data, rows=response.data.get('results', []),
#                         headers={'pk': 'ID', 'name': 'Name', 'code': 'Code',
#                                  'cloud_provider': 'Provider', 'zone': 'Zone'},
#                         is_succeed=response.is_succeed)
#
#
#

class KubeMetricMonitor(Controller):
    class Meta:
        label = 'metrics'
        stacked_type = 'nested'
        stacked_on = 'base'
        description = 'this is the kube metric monitor controller namespace'

    @ex(
        help='Metrics List Command',
        arguments=[
            (['cluster'], {
                'help': 'Application name',
                'action': 'store',
            }),
            (['namespace'], {
                'help': 'Application slug',
                'action': 'store',
            }),
            PaginationEnum.ARG
        ],
    )
    def list(self):
        response = self.app.client.get_metrics(
            cluster=self.app.pargs.cluster,
            namespace=self.app.pargs.namespace,
            qs={"page": getattr(self.app.pargs, PaginationEnum.KEY)}
        )
        self.app.render(data=response.data, rows=response.data,
                        headers={
                            'name': 'Name',
                            'containers': 'Containers',
                            'status': 'Status',
                        },
                        is_succeed=response.is_succeed)
